require 'cabeza-de-termo/assets/bundles/bundle'
require 'cabeza-de-termo/assets/errors/bundle-not-found-error'

module CabezaDeTermo
	module Assets
		# A collection of named bundles.
		#
		class BundlesLibrary
			# Initialize the instance
			def initialize
				@bundles = Hash.new
			end

			# Answer whether the library has the bundle named name or not.
			def has_asset_named?(name)
				@bundles.key? name
			end

			# Answer the bundle named name or raise an error if not found.
			def bundle_named(name)
				raise_bundle_not_found_error(name) unless has_asset_named?(name)
				@bundles[name]
			end

			# Define a bundle named name and allow to :include assets and :require another bundles on it.
			def bundle(name, &block)
				@bundles[name] = new_bundle
				CdT.bind_block_evaluation_to @bundles[name], &block
			end

			# Answer a new bundle
			def new_bundle
				Bundle.new
			end

			def raise_bundle_not_found_error(name)
				raise BundleNotFoundError.new("Asset bundle '#{name}' not found.")
			end
		end
	end
end