module CabezaDeTermo
	module Assets
		# This class is a wrapper on a Hanami::Views::LayoutScope object to delegate the 
		# :collect_stylesheets_with and :collect_javascripts_with to the actual layout_scope.
		class HanamiRenderingScope
			# Class methods

			# Answer a new adaptor on the layout_scope
			def self.on(layout_scope)
				new(layout_scope)
			end

			# Collect the stylesheets from the layout_scope
			def self.stylesheets_from(layout_scope)
				Library.stylesheets_for (self.on layout_scope)
			end

			# Collect the javascripts from the layout_scope
			def self.javascripts_from(layout_scope)
				Library.javascripts_for (self.on layout_scope)
			end

			# Collect the stylesheets from the layout_scope and iterate over each stylesheet
			def self.each_stylesheet_from(layout_scope, &block)
				stylesheets_from(layout_scope).each(&block)
			end

			# Collect the javascripts from the layout_scope and iterate over each javascript
			def self.each_javascript_from(layout_scope, &block)
				javascripts_from(layout_scope).each(&block)
			end

			# Instance methods

			def initialize(layout_scope)
				@layout_scope = layout_scope
			end

			# Delegate the :collect_stylesheets_with method to the actual Hanami::LayoutScope
			def collect_stylesheets_with(assets_collector)
				collect_stylesheets_from_layout_with assets_collector
				collect_stylesheets_from_view_with assets_collector
			end

			# Delegate the :collect_javascripts_with method to the actual Hanami::LayoutScope
			def collect_javascripts_with(assets_collector)
				collect_javascripts_from_layout_with assets_collector
				collect_javascripts_from_view_with assets_collector
			end

			protected
			# Collect the stylesheets from the layout on the @layout_scope, if present.
			def collect_stylesheets_from_layout_with(assets_collector)
				return unless @layout_scope.respond_to?(:layout_stylesheets)
				@layout_scope.layout_stylesheets assets_collector
			end

			# Collect the stylesheets from the view on the @layout_scope, if present.
			def collect_stylesheets_from_view_with(assets_collector)
				return unless @layout_scope.respond_to?(:view_stylesheets)
				@layout_scope.view_stylesheets assets_collector
			end

			# Collect the javascripts from the layout on the @layout_scope, if present.
			def collect_javascripts_from_layout_with(assets_collector)
				return unless @layout_scope.respond_to?(:layout_javascripts)
				@layout_scope.layout_javascripts assets_collector
			end

			# Collect the javascripts from the view on the @layout_scope, if present.
			def collect_javascripts_from_view_with(assets_collector)
				return unless @layout_scope.respond_to?(:view_javascripts)
				@layout_scope.view_javascripts assets_collector
			end
		end
	end
end